# Generated by Django 2.1.7 on 2019-03-24 22:50

from django.conf import settings
from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):

    dependencies = [("todo", "0007_auto_update_created_date")]

    operations = [
        migrations.AddField(
            model_name="comment",
            name="email_from",
            field=models.CharField(blank=True, max_length=320, null=True),
        ),
        migrations.AddField(
            model_name="comment",
            name="email_message_id",
            field=models.CharField(blank=True, max_length=255, null=True),
        ),
        migrations.AlterField(
            model_name="comment",
            name="author",
            field=models.ForeignKey(
                blank=True,
                null=True,
                on_delete=django.db.models.deletion.CASCADE,
                to=settings.AUTH_USER_MODEL,
            ),
        ),
        migrations.AlterField(
            model_name="task",
            name="created_by",
            field=models.ForeignKey(
                null=True,
                on_delete=django.db.models.deletion.CASCADE,
                related_name="todo_created_by",
                to=settings.AUTH_USER_MODEL,
            ),
        ),
        migrations.AlterUniqueTogether(
            name="comment", unique_together={("task", "email_message_id")}
        ),
    ]
